library(vars)
library(foreign)
library(rjags)
library(coda)
library(lme4)
library(DataCombine)

mldata=read.table("multileveldata.txt", header=TRUE)

nr=6 ##number of random parameters
tt= 50 ## number of time points
np= 50 #number of subjects

##fit data ML in lme4###

out1=lmer(Y1 ~  Y1lagcent + Y2lagcent + (1+Y1lagcent + Y2lagcent|ppn), data=mldata) ###estimate first univariate model in lme4
out2=lmer(Y2 ~ Y1lagcent + Y2lagcent + (1+Y1lagcent + Y2lagcent|ppn), data=mldata) ###estimate second univariate model in lme4

####Calculate ML prior input####
###Read out the estimated variances from the lme4 output and put it into matrix W, for the prior specification of the 
###covariance matrix of the random parameters###
###see Schuurman, Grasman and Hamaker###

wmat1=VarCorr(out1)
wmat2=VarCorr(out2)

wvars1= c(wmat1$ppn[1],wmat1$ppn[5],wmat1$ppn[9])
wvars2= c(wmat2$ppn[1],wmat2$ppn[5],wmat2$ppn[9])



df=6 ###degrees of freedom for the Wishart in WinBUGS 
W=matrix(0,6,6) #W is the scale matrix for the Wishart in WinBUGS
W[1,1]= df*wvars1[1]
W[2,2]= df*wvars2[1]
W[3,3]= df*wvars1[3] 
W[4,4]= df*wvars2[2] 
W[5,5]= df*wvars1[2]
W[6,6]= df*wvars2[3] 


#######Call Winbugs and fit the full model#####

f=mldata[,2:3]
dat=list("np"=np,"f"=f,"tt"=tt,"W"=W)###data for winbugs

###specify some starting values, for example: ####
stbpre1 = matrix(c(.01, 0, 0, 0, 0, 0,
                   0,.01,0, 0,0,0,
                   0,0, .01, 0 ,0,0,
                   0, 0, 0,0.01, 0, 0,
                   0, 0, 0, 0, .01, 0,
                   0, 0, 0, 0, 0, .01),6,6,byrow=TRUE)
stbpre2 = matrix(c(.015, 0, 0, 0, 0, 0,
                   0,.015,0, 0,0,0,
                   0,0, .015, 0 ,0,0,
                   0, 0, 0,0.015, 0, 0,
                   0, 0, 0, 0, .015, 0,
                   0, 0, 0, 0, 0, .015),6,6,byrow=TRUE)
stbpre3 = matrix(c(.005, 0, 0, 0, 0, 0,
                   0,.005,0, 0,0,0,
                   0,0, .005, 0 ,0,0,
                   0, 0, 0,0.005, 0, 0,
                   0, 0, 0, 0, .005, 0,
                   0, 0, 0, 0, 0, .005),6,6,byrow=TRUE)

phistart = matrix(0,np,nr)
phistart2 = phistart + .1
phistart3 = phistart + .2

inits1 <- list( bmu=c(0,0,0,0,1,1),  b=phistart, bpre=stbpre1) ###for chain1
inits2 <- list( bmu=c(0.1,0.1,0.1,0.1,2,2),  b=phistart2,  bpre=stbpre2) ###for chain 2
inits3 <- list( bmu=c(0.2,0.2,0.2,0.2,1.5,1.5),  b=phistart3,  bpre=stbpre3) ###for chain 3

ins <- list(inits1,inits2) ###inits that are provided to winbugs, list all three inits if running 3 chains

jags <- jags.model('MLVAR1model.txt',
                   data = dat,
                   inits=ins,
                   n.chains = 2) ###compile and initialize jags AR model


update(jags, 20000) ####burnin

samps=coda.samples(jags,
                   c('bmu','bcov', 'Qvar','Qcor', 'b'),
                   20000) ### take samples next to burnin

#dic.samps=dic.samples(jags,
#                     20000) ##take samples for DIC


plot(samps,trace=FALSE) ##plot densities

summary(samps) ##results parameters estimates 
summary(samps)$quantiles["bmu[1]",]

gelman.plot(samps) ###plot gelman rubin stats

plot(samps,density=FALSE) ##traceplots


